from django.core.paginator import Paginator
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404, render
from tumbledore.models import *


def index(request, mount_point):
    page = int(request.GET.get('page', '1'))
    tumblelog = get_object_or_404(Tumblelog, mount_on=mount_point, is_active=True)
    queryset = tumblelog.posts.all() if request.user.is_staff else tumblelog.posts.published()
    if tumblelog.sort_posts_by is 'C':
        queryset = queryset.order_by(['sort_order'])

    object_list = Paginator(queryset, tumblelog.posts_per_page).page(int(page))
    for obj in object_list:
        # set permalink
        permalink = reverse('tumble_post', urlconf='tumbledore.urls', args=[obj.mount_on, obj.slug])
        obj = obj.__dict__.update(permalink=permalink)
        # override any custom variables
        obj = obj.__dict__.update(**obj.custom_data)

    return render(request, '%s/index.html' % tumblelog.theme, {
        'tumblelog': tumblelog,
        'object_list': object_list,
        'widgets': _get_widgets_for(tumblelog),
        })


def post(request, mount_point, slug):
    tumblelog = get_object_or_404(Tumblelog, mount_on=mount_point, is_active=True)
    query_args = {
        'slug': slug,
        'tumblelog_id': tumblelog.id,
        'is_published': True,
        }
    if 'preview' in request.GET.keys():
        del query_args['is_published']
    print query_args
    try:
        post = TumblelogPost.objects.filter(**query_args)[0]
    except IndexError:
        raise Http404

    return render(request, '%s/post.html' % tumblelog.theme, {
        'tumblelog': tumblelog,
        'post': post,
        'widgets': _get_widgets_for(tumblelog),
        })


def widget(request, widget_id):
    widget = get_object_or_404(TumblelogWidget, pk=widget_id)

    return HttpResponse(widget.as_javascript, content_type="text/javascript")


def _get_widgets_for(tumblelog):
    return [placement.widget for placement in tumblelog.placement_set.all()]



from django.contrib import admin

from tumbledore.models import *


class TumblelogPostInline(admin.StackedInline):
    model = TumblelogPost
    list_editable = ('is_published', 'is_sticky')
    prepopulated_fields = {"slug": ("title",), }


class TumblelogWidgetPlacementInline(admin.TabularInline):
    model = TumblelogWidgetPlacement


class TumblelogAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'mount_on', 'theme', 'is_active', 'sort_posts_by', 'created_at')
    list_filter = ('theme', 'is_active')
    list_editable = ('theme', 'is_active', 'sort_posts_by')
    date_hierarchy = 'created_at'
    inlines = (TumblelogWidgetPlacementInline, TumblelogPostInline)
    save_on_top = True
    fieldsets = (
        (None, {
            'fields': ('name', 'mount_on', 'theme', 'posts_per_page')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('description', 'extra_styles', 'extra_scripts')
        }),
    )
admin.site.register(Tumblelog, TumblelogAdmin)


class TumblelogPostAdmin(admin.ModelAdmin):
    list_display = ('title', 'author', 'tumblelog', 'is_published', 'is_sticky', 'created_at', 'published_at')
    list_filter = ('author', 'tumblelog', 'is_published', 'is_sticky')
    list_editable = ('is_published', 'is_sticky')
    date_hierarchy = 'published_at'
    prepopulated_fields = {"slug": ("title",), }
    save_on_top = True
    fieldsets = (
        (None, {
            'fields': ('title', 'slug', 'tumblelog', 'author', 'content',
                       'is_published', 'is_sticky', 'has_permalink', 'published_at')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('excerpt', 'sort_order', 'custom_data')
        }),
    )
admin.site.register(TumblelogPost, TumblelogPostAdmin)


class TumblelogWidgetAdmin(admin.ModelAdmin):
    list_display = ('name', 'description', 'created_at')
    date_hierarchy = 'created_at'
    prepopulated_fields = {"slug": ("name",)}
    save_on_top = True
admin.site.register(TumblelogWidget, TumblelogWidgetAdmin)


class TumblelogWidgetPlacementAdmin(admin.ModelAdmin):
    list_display = ('widget', 'tumblelog', 'order')
    list_editable = ('order',)
admin.site.register(TumblelogWidgetPlacement, TumblelogWidgetPlacementAdmin)



from django.conf.urls.defaults import *

urlpatterns = patterns('tumbledore.views',
    url(r'^tumblelog/widget/(?P<widget_id>[\d]+).js$', 'widget', name='tumble_widget'),
    url(r'^(?P<mount_point>[\d\w\/_\-]*)/post/(?P<slug>[\w\d_\-]+)/$', 'post', name='tumble_post'),
    url(r'^(?P<mount_point>[\d\w\/_\-]*)/$', 'index', name='tumble_index'),
)



import datetime
import glob
import os

from django.conf import settings
from django.db import models
from django.utils.html import escapejs
from jsonfield import JSONField


DEFAULT_POSTS_PER_PAGE = 10

THEME_PATHS = glob.glob(os.path.join(os.path.dirname(__file__),
                                     'templates', 'tumbledore', 'themes', '*'))
for location in settings.TEMPLATE_DIRS:
    THEME_PATHS += glob.glob(os.path.join(location, 'tumbledore', 'themes', '*'))
TUMBL_THEME_CHOICES = [(path, path.split('/')[-1]) for path in THEME_PATHS]

SORT_BY_CHOICES = (
    ('D', 'Date',),
    ('O', 'Order',),
)


class Tumblelog(models.Model):
    name = models.CharField(max_length=200,
                            help_text='This will display in the header area of your tumblelog.')
    description = models.TextField(blank=True, default='',
                                   help_text='This will display below the name.')
    mount_on = models.CharField(max_length=200, blank=True, default='',
                                unique=True, db_index=True,
                                help_text="The root url of your tumblelog.")
    theme = models.CharField(max_length=255, choices=TUMBL_THEME_CHOICES)
    posts_per_page = models.IntegerField(default=DEFAULT_POSTS_PER_PAGE)
    sort_posts_by = models.CharField(max_length=1, choices=SORT_BY_CHOICES, default='D')
    widgets = models.ManyToManyField('TumblelogWidget', through='TumblelogWidgetPlacement')
    extra_styles = models.TextField(blank=True, default='',
                                    help_text='Inserted before the close of the head tag.')
    extra_scripts = models.TextField(blank=True, default='',
                                     help_text='Inserted before the close of the body tag.')
    is_active = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        pass

    def __unicode__(self):
        return unicode(self.name)

    def save(self, **kwargs):
        self.mount_on = self.mount_on.strip('/')

        super(Tumblelog, self).save(**kwargs)


class TumblelogPostManager(models.Manager):
    def published(qset):
        return qset.filter(is_published=True)

    def drafts(qset):
        return qset.filter(is_published=False)


class TumblelogPost(models.Model):
    title = models.CharField(max_length=200)
    slug = models.SlugField(max_length=200, db_index=True)
    tumblelog = models.ForeignKey(Tumblelog, related_name='posts',
                                  verbose_name='Post in')
    author = models.CharField(max_length=200, blank=True, default='',
                              help_text='HTML is allowed.')
    content = models.TextField(blank=True, default='')
    excerpt = models.TextField(blank=True, default='',
                               help_text='Only needed if this post has a permalink.')
    sort_order = models.IntegerField(blank=True, default=0,
                                     help_text='Set this if your tumblelog is using custom ordering.')
    custom_data = JSONField(default="{}",
                                   help_text='As JSON; will override context variables, e.g., permalink')
    is_published = models.BooleanField(default=False)
    is_sticky = models.BooleanField(default=False)
    has_permalink = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)
    published_at = models.DateTimeField(blank=True, null=True, db_index=True)

    objects = TumblelogPostManager()

    class Meta:
        ordering = ('-is_sticky', '-published_at')
        unique_together = ('slug', 'tumblelog')

    def __unicode__(self):
        return unicode("%s, by %s" % (self.title, self.author))

    def save(self, **kwargs):
        if self.is_published and self.published_at is None:
            self.published_at = datetime.datetime.now()

        super(TumblelogPost, self).save(**kwargs)


class TumblelogWidget(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField(max_length=200, unique=True, db_index=True)
    description = models.TextField(blank=True, default='')
    content = models.TextField(blank=True, default='')
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)

    def __unicode__(self):
        return unicode(self.name)

    @property
    def as_javascript(self):
        return ('document.write("%s");' % escapejs(self.content))


class TumblelogWidgetPlacement(models.Model):
    tumblelog = models.ForeignKey(Tumblelog, related_name='placement_set')
    widget = models.ForeignKey(TumblelogWidget, related_name='placement_set')
    order = models.IntegerField(default=1)

    class Meta:
        ordering = ('order',)

    def __unicode__(self):
        return unicode('%s on %s' % (self.widget.name, self.tumblelog.name))

    def save(self, **kwargs):
        # increment pre-existing widget with this order, if it exists
        try:
            conflict = TumblelogWidgetPlacement.objects.get(tumblelog=self.tumblelog, order=self.order)
            conflict.order += 1
            conflict.save()
        except TumblelogWidgetPlacement.DoesNotExist:
            pass
        super(TumblelogWidgetPlacement, self).save(**kwargs)



#!/usr/bin/env python
# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

from tumbledore import __appname__, __version__
from setuptools import setup

long_description = open('README.md').read()

setup(
    name       = __appname__,
    version    = __version__,
    packages   = [ 'tumbledore', ],

    author       = "Dan Drinkard",
    author_email = "ddrinkard@sunlightfoundation.com",

    long_description = long_description,
    description      = 'A simple, barebones tumblelog implementation for your django site',
    license          = "BSD",
    url              = "https://github.com/sunlightlabs/django-tumbledore",

    platforms        = ['any'],
    install_requires = ['django >= 1.4', 'jsonfield', ]
)



# Copyright (c) Sunlight Labs, 2012 under the terms and conditions
# of the LICENSE file.

"""
.. module:: tumbledore
    :synopsis: A Lightweight Tumblelog App

Django-Tumbledore __init__ entry. Does little else than define an
appname and version.
"""

__appname__ = "tumbledore"
__version__ = "0.1.3"
